Package: ADAM
Type: Package
Title: ADAM: Activity and Diversity Analysis Module
Version: 1.29.0
Date: 2023-04-10
Authors@R: 
    c(person(given = "André Luiz",
                 family = "Molan",
                 role = "aut",
                 email = "andre.molan@unesp.br"),
      person(given = "Giordano Bruno",
                 family = "Sanches Seco",
                 role = "ctb"),
      person(given = "Agnes",
                family = "Takeda",
                role = "ctb"),
      person(given = "Jose",
                family = "Rybarczyk Filho",
                role = c("ctb", "cre", "ths"),
                email = "jose.luiz@unesp.br")
			 )
Maintainer: Jose Luiz Rybarczyk Filho <jose.luiz@unesp.br>
Description: ADAM is a GSEA R package created to group a set of genes from
    comparative samples (control versus experiment) belonging to different
    species according to their respective functions (Gene Ontology and KEGG
    pathways as default) and show their significance by calculating p-values
    referring togene diversity and activity. Each group of genes is called GFAG
    (Group of Functionally Associated Genes). 
Depends: R(>= 3.5), stats, utils, methods
Imports: Rcpp (>= 0.12.18), GO.db (>= 3.6.0), KEGGREST (>= 1.20.2),
        knitr, pbapply (>= 1.3-4), dplyr (>= 0.7.6), DT (>= 0.4),
        stringr (>= 1.3.1), SummarizedExperiment (>= 1.10.1)
Suggests: testthat, rmarkdown, BiocStyle
LinkingTo: Rcpp
SystemRequirements: C++11
Encoding: UTF-8
RoxygenNote: 6.1.0
License: GPL (>= 2)
biocViews: GeneSetEnrichment, Pathways, KEGG, GeneExpression,
        Microarray
VignetteBuilder: knitr
LazyLoad: yes
NeedsCompilation: yes
git_url: https://git.bioconductor.org/packages/ADAM
git_branch: devel
git_last_commit: 8f82406
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.24
Date/Publication: 2026-04-28
Packaged: 2026-04-28 21:10:30 UTC; biocbuild
Author: André Luiz Molan [aut],
  Giordano Bruno Sanches Seco [ctb],
  Agnes Takeda [ctb],
  Jose Rybarczyk Filho [ctb, cre, ths]
