# Generated by roxygen2: do not edit by hand

export(ADAnalysis)
export(GFAGAnalysis)
import(GO.db)
import(KEGGREST)
import(Rcpp)
import(knitr)
import(methods)
import(pbapply)
importFrom(DT,datatable)
importFrom(Rcpp,evalCpp)
importFrom(SummarizedExperiment,assay)
importFrom(dplyr,arrange)
importFrom(dplyr,filter)
importFrom(dplyr,mutate)
importFrom(stats,fisher.test)
importFrom(stats,p.adjust)
importFrom(stats,wilcox.test)
importFrom(stringr,str_split)
importFrom(utils,installed.packages)
importFrom(utils,read.table)
importFrom(utils,write.table)
useDynLib(ADAM)
