setClass("ECGMainData",
        slots = c(ComparisonID = "vector",
                ExpressionData = "data.frame",
                MinGene = "integer",
                MaxGene = "integer",
                SeedNumber = "numeric",
                BootstrapNumber = "integer",
                PCorrection = "numeric",
                DBSpeciesFunctionsSample = "list",
                DBSpeciesFunctionsRaw = "list",
                PCorrectionMethod = "character",
                WilcoxonTest = "logical",
                FisherTest = "logical",
                AnalysisDomain = "character",
                GeneIdentifier = "character"))

